package core.entities.timemanager;

import core.api.models.timemanager.history.TaskgroupActivityInfo;
import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.entities.User;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.util.*;

@Entity
@Table(name= "taskgroups")
public class Taskgroup {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long taskgroupID;

    @NotBlank
    @Column(name = "name", length = 255)
    private String taskgroupName;

    @OneToOne
    @JoinColumn(name = "taskgroupuser", nullable = false)
    private User user;

    @OneToMany(mappedBy = "parent", cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
    private Set<Taskgroup> children;

    @ManyToOne
    @JoinColumn(name = "parent_id")
    private Taskgroup parent;


    @OneToMany(mappedBy = "taskgroup", cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
    private Set<Task> tasks;

    public Taskgroup(String taskgroupName, User user) {
        this.taskgroupName = taskgroupName;
        this.user = user;
    }

    public Taskgroup() {
    }

    public long getTaskgroupID() {
        return taskgroupID;
    }

    public void setTaskgroupID(long taskgroupID) {
        this.taskgroupID = taskgroupID;
    }

    public String getTaskgroupName() {
        return taskgroupName;
    }

    public void setTaskgroupName(String taskgroupName) {
        this.taskgroupName = taskgroupName;
    }

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public Set<Taskgroup> getChildren() {
        return children;
    }

    public void setChildren(Set<Taskgroup> children) {
        this.children = children;
    }

    public Taskgroup getParent() {
        return parent;
    }

    public void setParent(Taskgroup parent) {
        this.parent = parent;
    }

    public Set<Task> getTasks() {
        return tasks;
    }

    public void setTasks(Set<Task> tasks) {
        this.tasks = tasks;
    }

    public static List<Taskgroup> getAncestorList(Taskgroup taskgroup) {
        List<Taskgroup> ancestors = new ArrayList<>();
        Taskgroup currentTaskgroup = taskgroup;
        ancestors.add(taskgroup);
        while(currentTaskgroup.parent != null) {
            ancestors.add(currentTaskgroup.parent);
            currentTaskgroup = currentTaskgroup.parent;
        }

        Collections.reverse(ancestors);
        return ancestors;
    }

    public List<Task> getActiveTasks() {
        List<Task> tasks = new LinkedList<>();
        for(Task task : this.tasks) {
            if(!task.isFinished()) {
                tasks.add(task);
            }
        }
        return tasks;
    }

    public int getAmountOfActiveTasks() {
        int activeAmountTasks = getActiveTasks().size();
        for(Taskgroup taskgroup : children) {
            activeAmountTasks += taskgroup.getAmountOfActiveTasks();
        }
        return activeAmountTasks;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Taskgroup taskgroup = (Taskgroup) o;
        return taskgroupID == taskgroup.taskgroupID;
    }

    @Override
    public int hashCode() {
        return Objects.hash(taskgroupID);
    }

    public List<TaskgroupActivityInfo> calcActivityInfo(boolean includeChildTasks, LocalDate start, LocalDate end) {
        HashMap<LocalDate, Integer> activityInfos = new HashMap<>();
        if(includeChildTasks) {

            Queue<Taskgroup> queue = new LinkedList<>(children);
            while(!queue.isEmpty()) {
                Taskgroup childTraskgroup = queue.poll();
                LocalDate currentDate = start;
                while(!currentDate.isAfter(end)) {
                    int activeMinutes = 0;
                    for(Task task : childTraskgroup.getTasks()) {
                        activeMinutes += task.calcOverallActivityInfo(currentDate);
                    }

                    if(activityInfos.containsKey(currentDate)) {
                        activityInfos.put(currentDate, activityInfos.get(currentDate) + activeMinutes);
                    } else {
                        activityInfos.put(currentDate, activeMinutes);
                    }

                    currentDate = currentDate.plusDays(1);
                }


                queue.addAll(childTraskgroup.getChildren());
            }
        }

        LocalDate currentDate = start;
        while(!currentDate.isAfter(end)) {
            int activeMinutes = 0;
            for(Task task : tasks) {
                activeMinutes += task.calcOverallActivityInfo(currentDate);
            }

            if(activityInfos.containsKey(currentDate)) {
                activityInfos.put(currentDate, activityInfos.get(currentDate) + activeMinutes);
            } else {
                activityInfos.put(currentDate, activeMinutes);
            }

            currentDate = currentDate.plusDays(1);
        }

        List<TaskgroupActivityInfo> taskgroupActivityInfos = new ArrayList<>();
        for(Map.Entry<LocalDate, Integer> entry : activityInfos.entrySet()) {
            taskgroupActivityInfos.add(new TaskgroupActivityInfo(entry.getValue(), entry.getKey()));
        }
        return taskgroupActivityInfos;
    }
}
